__author__ = "Dmitry Philippov"

import sys, os
import logging
                                           
class XMLWriter:
    def __init__(self, path=None):
        self.contest_id = "ru.ifmo.neerc.io.2015-2016.20160312."
        self.problem_id = "triangles"


        self.log_file = "description.log"
        with open(self.log_file, "w"):
            pass
        logging.basicConfig(filename=self.log_file, level=logging.DEBUG)
        self.parameters = ["folder", "subtask", "points", "comment"]
        self.special_parameters = ["comment_subtask", "bonus_subtask"]
        self.description_path = "tests.desc"
        if path is not None:    
            self.description_path = path
        if not os.path.exists(self.description_path):
            print("Can\'t load description file from path \"%s\"" % (self.description_path))
            sys.exit(0)
        self.description_file = open(self.description_path, "r")
        self.subtask_special_parameter = {}
        self.tests = {}

    def _get_special_parameter(self, line):
        for parameter in self.special_parameters:
            if line.startswith(parameter):
                return parameter
        return None

    def _is_special_line(self, line):
        return self._get_special_parameter(line) is not None

    def _is_digit(self, c):
        return ord("0") <= ord(c) <= ord("9")

    def _subtask_number(self, line, prefix):
        subtask_number = 0
        index = len(prefix)
        while (index < len(line)) and (self._is_digit(line[index])):
            subtask_number = subtask_number * 10 + (ord(line[index]) - ord("0"))
            index += 1
        return subtask_number

    def _update_special_parameter(self, subtask_number, special_parameter, value):
        logging.info("Updating special parameter: %d %s %s" % (subtask_number, special_parameter, value))
        subtask_number = "subtask" + str(subtask_number)
        if self.subtask_special_parameter.get(subtask_number) is None:
            self.subtask_special_parameter[subtask_number] = {}
        self.subtask_special_parameter[subtask_number][special_parameter] = value

    def _update_test(self, subtask_number, test_values):
        #logging.info("Updating parameter: %d" % subtask_number)
        if self.tests.get(subtask_number) is None:
            self.tests[subtask_number] = []
        self.tests[subtask_number].append(test_values)
        
    def process(self):
        lines = self.description_file.readlines()
        for line in lines:
            line = line.rstrip()
            if self._is_special_line(line):
                special_parameter = self._get_special_parameter(line)
                subtask_number = self._subtask_number(line, special_parameter)
                self._update_special_parameter(subtask_number,
                                               special_parameter,
                                               line[line.find("=") + 1:]
                                               )
            else:
                test_name = line[:line.find(" ") - 1]
                logging.info("Processing test %s" % test_name)
                test_values = {}
                for parameter in self.parameters:
                    index = line.find(parameter + "=")
                    assert(index != -1)
                    index += len(parameter) + 1
                    test_value = ""
                    if parameter == "comment":
                        test_value = line[index:]
                    else:
                        while line[index] != ",":
                            test_value += line[index]
                            index += 1
                    test_values[parameter] = test_value
                subtask_number = self._subtask_number(test_values["subtask"], "subtask") 
                self._update_test(subtask_number, test_values)

    def write_xml(self, output_file):
        #head of xml file
        print("<?xml version=\"1.0\" encoding=\"UTF-8\"?>", file=output_file)
        print("<problem\n\
    version=\"1.0\"\n\
    id =\"%s%s\"\n\
>\n\
    <judging>\n\
        <script type=\"%sioi\">"
% (self.contest_id, self.problem_id, "%"), file=output_file)

        #updating list of tests: tests[folder][subtask] = [test_1, test_2, ...]
        #folder = preliminary | tests
        #subtask = subtask0 | subtask1 | ...

        tests = {}
        for subtask_name in self.tests:
            for test in self.tests[subtask_name]:
                folder = test["folder"]
                subtask = test["subtask"]
                if tests.get(folder) is None:
                    tests[folder] = {}
                if tests[folder].get(subtask) is None:
                    tests[folder][subtask] = []
                tests[folder][subtask].append(test)

        #preliminary testset
        print("\
            <testset\n\
                name=\"preliminary\"\n\n\
                input-name  =\"%s.in\"\n\
                output-name =\"%s.out\"\n\
                input-href  =\"preliminary/##\"\n\
                answer-href =\"preliminary/##.a\"\n\n\
                time-limit = \"2s\"\n\
                memory-limit =\"268435456\"\n\
            >" % (self.problem_id, self.problem_id), file=output_file)

        assert(len(tests["preliminary"]) == 1)
        subtask = list(tests["preliminary"].items())[0][0]
        for test in tests["preliminary"][subtask]:
            print("\
                <test points=\"%s\" comment=%s/>" % (test["points"], test["comment"]\
                  ), file=output_file)
        print("\
            </testset>\n", file=output_file)

        #main testset
        print("\
            <testset\n\
                name =\"main\"\n\
                input-name  =\"%s.in\"\n\
                output-name =\"%s.out\"\n\
                input-href  =\"tests/##\"\n\
                answer-href =\"tests/##.a\"\n\n\
                time-limit =\"2s\"\n\
                memory-limit =\"268435456\"\n\
            >" % (self.problem_id, self.problem_id), file=output_file)

        subtasks = []
        for subtask in tests["tests"]:
            subtasks.append(subtask)
        subtasks.sort()
        for subtask in subtasks:
            print('\
                <test-group\n\
                    comment = %s\n\
                    score = \"sum\"\n\
                    feedback = \"outcome\"\n\
                    group-bonus = \"%s\"\n\
                >' % (self.subtask_special_parameter[subtask]["comment_subtask"],\
                      self.subtask_special_parameter[subtask]["bonus_subtask"]\
                      ), file=output_file)
            for test in tests["tests"][subtask]:
                print('\
                    <test points = \"%s\" comment = %s/>\
                ' % (test["points"], test["comment"]), file=output_file)
            print("\
                </test-group>\n", file=output_file)
        print("\
            </testset>\n", file=output_file)

        #tail of xml file
        print("\
            <verifier type=\"%testlib\">\n\
                <binary executable-id =\"java.check\" file =\"check.jar\" />\n\
            </verifier>\n\
        </script>\n\
    </judging>\n\
</problem>", file=output_file)


    def create_xml(self, path):
        #try:
        logging.info("Started processing description file") 
        self.process()
        logging.info("Finished processing")
        output_file = open(path, "w", encoding="utf-8")
        logging.info("Writing xml to %s" % path)
        self.write_xml(output_file)
        logging.info("Finishing")
        output_file.close()
        #except Exception as e:
        #exc_type, exc_value, exc_traceback = sys.exc_info()
        #need better message
        #print("Unknown error occured while processing and writing xml: " + str(e))

if len(sys.argv) < 2:
    writer = XMLWriter()
    writer.create_xml("problem.xml")
else:
    writer = XMLWriter(sys.argv[1])
    path = sys.argv[1]
    index = max(path.rfind('/'), path.rfind('\\'))
    path = path.replace(path[index + 1:], 'problem.xml')
    writer.create_xml(path)